from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand

from getpass import getpass
from signal import SIGINT, signal
from subprocess import run
from time import sleep

from rahavard import (
    abort,
    colorize,
    get_command,
    get_command_log_file,
    keyboard_interrupt_handler,
    save_log,
)


ACTION_OPTIONS = [
    'dumpdata',
    'collectstatic',
    'check-deploy',

    'renew',
    'update',
    'check-trace',
]


signal(SIGINT, keyboard_interrupt_handler)


class Command(BaseCommand):
    help = 'Actions'

    def add_arguments(self, parser):
        parser.add_argument(
            '-a',
            '--action',
            default=None,
            type=str,
            help=f'action (options: {",".join(ACTION_OPTIONS)})',
        )

    def handle(self, *args, **kwargs):
        action = kwargs.get('action')

        if not action:
            return abort(self, 'no action specified')

        if action not in ACTION_OPTIONS:
            return abort(self, 'invalid action')

        command  = get_command(full_path=__file__, drop_extention=True)
        log_file = get_command_log_file(f'{command}--{action}')
        ## .../actions--renew.log

        if action in ['dumpdata', 'collectstatic', 'check-deploy']:
            try:
                call_command(action)
            except Exception as exc:
                save_log(self, command, settings.HOST_NAME, log_file, f'action={action}: {exc!r}')
        ## -----------------------------------
        elif action == 'renew':
            cmd = run(
                'sudo certbot renew',
                shell=True,
                universal_newlines=True,
                capture_output=True,
            )
            cmd_ext_stts = cmd.returncode  ## 0/1/...
            cmd_output   = cmd.stdout.strip()
            cmd_error    = cmd.stderr.strip()

            if cmd_ext_stts == 0:  ## successful
                print(cmd_output)
            elif cmd_ext_stts:
                save_log(self, command, settings.HOST_NAME, log_file, cmd_error)
        ## -----------------------------------
        elif action == 'update':
            gh_username = input('github username: ').strip()
            gh_token    = getpass('github token: ').strip()

            repo_url = f'https://{gh_username}:{gh_token}@github.com/{gh_username}/{settings.PROJECT_SLUG}.git'
            branch = 'master'

            tuples = [
                ('Fetching...',            f'git -C {settings.PROJECT_DIR} fetch {repo_url} {branch}'),
                ('Pulling...',             f'git -C {settings.PROJECT_DIR} pull  {repo_url} {branch}'),
                ('Restarting apache24...', 'sudo service apache24 restart'),
            ]

            for index, (description, cmd) in enumerate(tuples, start=1):
                print(f'{index}/{len(tuples)} {description}')
                run(cmd, shell=True)
                print('-_'*30)
                sleep(.1)

        ## -----------------------------------
        elif action == 'check-trace':
            cmd = run(
                ## --connect-timeout and --max-time:
                ## https://unix.stackexchange.com/a/94612
                f'curl -v -X TRACE --connect-timeout 20 --max-time 60 {settings.TARCE_URL} 2>&1',
                shell=True,
                universal_newlines=True,
                capture_output=True,
            )
            cmd_ext_stts = cmd.returncode  ## 0/1/...
            cmd_output   = cmd.stdout.strip()
            cmd_error    = cmd.stderr.strip()

            if cmd_ext_stts == 0:  ## successful
                # print(cmd_output)

                ## NOTE 1. although the above cmd has finished successfully,
                ##         we still have to look for the sensitive information
                ##      2. the line containing the sensitive information looks like this, if
                ##         a. it's secure:
                ##            < Server: Apache
                ##         b. it's NOT secure:
                ##            < Server: Apache/1.2.33 (FreeBSD) OpenSSL/1.2.33 mod_wsgi/1.2.33 Python/1.2
                if any([
                    'Server: Apache/' in cmd_output,
                    'OpenSSL/'        in cmd_output,
                    'mod_wsgi/'       in cmd_output,
                    'Python/'         in cmd_output,
                    '(FreeBSD)'       in cmd_output,
                ]):
                    print(colorize(self, 'error', cmd_output))
                    save_log(self, command, settings.HOST_NAME, log_file, cmd_output, echo=False)
                else:
                    print(colorize(self, 'success', 'Everything is safe'))

            elif cmd_ext_stts:
                save_log(self, command, settings.HOST_NAME, log_file, cmd_output)



from django.conf import settings
from django.core.management.commands import dumpdata
# from django.core.management.base import CommandError

from datetime import datetime
from os import path, makedirs, remove
from time import sleep

from natsort import natsorted
from rahavard import (
    contains_ymd,
    get_list_of_files,
    to_tilda,
)


## https://stackoverflow.com/a/37755287/
class Command(dumpdata.Command):
    help = 'My Customized Version of the Original dumpdata Command'

    ## https://stackoverflow.com/a/78204755/
    def __init__(self, *args, **kwargs):
        kwargs['no_color'] = not kwargs.get('force_color', False)
        super().__init__(*args, **kwargs)

    def handle(self, *args, **options):

        ## dump --------------

        print('>>> Running customized dumpdata\n')

        if not path.exists(settings.FIXTURES_DIR):
            print(f'creating {settings.FIXTURES_DIR}')
            makedirs(settings.FIXTURES_DIR)

        ## include app slug (if any) in output_file
        apps_string = ''
        if args:
            if len(args) == 1:
                apps_string = '--app-'
            else:
                apps_string = '--apps-'

            for _ in args:
                apps_string = f'{apps_string}{_}-'
            apps_string = apps_string.rstrip('-')

        ymdhms = datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        extension = 'json'

        ## JUMP_1
        output_file = f'{settings.FIXTURES_DIR}/{settings.HOST_NAME}-{ymdhms}{apps_string}.{extension}'
        print(f'dumping to: {to_tilda(output_file)}')

        options['format'] = extension
        options['indent'] = 2
        options['output'] = output_file
        options['skip_checks'] = True
        options['use_natural_foreign_keys'] = True
        options['use_natural_primary_keys'] = True
        options['verbosity'] = 3
        options['exclude'] = [
            'admin',
            'messages',
            'sessions',

            ## commented because we also need
            ## auth.group and contenttypes.contenttype models
            ## in the fixtures
            # 'auth',
            # 'contenttypes',
        ]

        super().handle(*args, **options)


        ## rotate --------------

        print('getting list of already present fixtures')
        fixtures = get_list_of_files(directory=settings.FIXTURES_DIR, extension=extension)
        fixtures = natsorted([
            _ for _ in fixtures
            if all([
                ## exclude non-fixtures
                ## by making sure file names match pattern in JUMP_1
                settings.HOST_NAME in _,
                contains_ymd(_),
            ])
        ])
        print(f'  count: {len(fixtures)}')

        ## 24: crontab is configured to generate fixtures every hour
        ##     giving us 24 fixtures a day
        ## 90: backup fixtures for 90 days
        LAST_FIXTURES_TO_SKIP = 24 * 90  ## last n fixtures to skip removing

        to_be_removed = fixtures[:-LAST_FIXTURES_TO_SKIP]

        if to_be_removed:
            to_be_removed__len = len(to_be_removed)
            print(f'{to_be_removed__len} to be removed:')
            for idx, tbr in enumerate(to_be_removed, start=1):
                print(f' {idx}/{to_be_removed__len}: removing {tbr}')
                remove(tbr)
                sleep(.1)



from django.core.management.commands import check
# from django.core.management.base import CommandError


## https://stackoverflow.com/a/37755287/
class Command(check.Command):
    help = 'My Customized Version of the Original check Command'

    ## https://stackoverflow.com/a/78204755/
    def __init__(self, *args, **kwargs):
        kwargs['no_color'] = not kwargs.get('force_color', False)
        super().__init__(*args, **kwargs)

    def handle(self, *args, **options):
        print('>>> Running customized check')

        options['deploy'] = True
        options['verbosity'] = 3

        super().handle(*args, **options)



from django.conf import settings
from django.contrib.staticfiles.management.commands import collectstatic
# from django.core.management.base import CommandError


## https://stackoverflow.com/a/37755287/
class Command(collectstatic.Command):
    help = 'My Customized Version of the Original collectstatic Command'

    ## https://stackoverflow.com/a/78204755/
    def __init__(self, *args, **kwargs):
        kwargs['no_color'] = not kwargs.get('force_color', False)
        super().__init__(*args, **kwargs)

    def handle(self, *args, **options):
        print('>>> Running customized collectstatic\n')

        ## on remote, static and staticfiles directories are constantly changing
        ## so there may be important files present in staticfiles directory 
        ## but absent in static directory
        ## for example 'uploads' or 'dl' directory
        ## so, if we clear staticfiles
        ## only when we are on local machine
        ## everything is safe
        if settings.DEBUG:
            options['clear'] = True

        options['interactive'] = False
        options['ignore_patterns'] = [
            'dl/*',
        ]
        options['verbosity'] = 3

        super().handle(*args, **options)


###############################

## method 2
## the disadvantage with this method is I couldn't find a way to pass args.
## e.g. the command 'manage.py collectstatic --dry-run wont work'

# from django.core.management import call_command
# from django.core.management.base import BaseCommand, CommandError

## https://stackoverflow.com/a/75189544/
# class Command(BaseCommand):
#     help = 'My Customized Version of the Original collectstatic Command'

#     def handle(self, *args, **kwargs):
#         call_command(
#             'collectstatic',
#             ignore_patterns=['dl/*'],
#             clear=True,
#             interactive=False,
#             **kwargs,
#         )
